from multiprocessing.pool import ThreadPool
from typing import Any

from aiohttp import ClientTimeout
from kubernetes_asyncio.client import rest as rest
from kubernetes_asyncio.client.configuration import Configuration as Configuration
from kubernetes_asyncio.client.exceptions import (
    ApiException as ApiException,
    ApiValueError as ApiValueError,
)

class ApiClient:
    PRIMITIVE_TYPES: list[Any]
    NATIVE_TYPES_MAPPING: dict[str, Any]
    configuration: Configuration
    pool_threads: int
    rest_client: rest.RESTClientObject
    default_headers: dict[str, str]
    cookie: str
    client_side_validation: bool
    def __init__(
        self,
        configuration: Configuration | None = None,
        header_name: str | None = None,
        header_value: str | None = None,
        cookie: str | None = None,
        pool_threads: int = 1,
    ) -> None: ...
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc_value, traceback) -> None: ...
    async def close(self) -> None: ...
    @property
    def pool(self) -> ThreadPool: ...
    @property
    def user_agent(self) -> str | None: ...
    @user_agent.setter
    def user_agent(self, value: str) -> None: ...
    def set_default_header(self, header_name: str, header_value: str) -> None: ...
    def sanitize_for_serialization(self, obj: Any) -> Any: ...
    def deserialize(self, response: rest.RESTResponse, response_type: Any): ...
    def call_api(
        self,
        resource_path: str,
        method: str,
        path_params: Any | None = None,
        query_params: Any | None = None,
        header_params: dict | None = None,
        body: Any | None = None,
        post_params: dict | None = None,
        files: Any | None = None,
        response_types_map: dict | None = None,
        auth_settings: list | None = None,
        async_req: bool | None = None,
        _return_http_data_only: bool | None = None,
        collection_formats: dict | None = None,
        _preload_content: bool = True,
        _request_timeout: float | tuple[float] | ClientTimeout | None = None,
        _host: str | None = None,
        _request_auth: str | None = None,
    ) -> Any: ...
    def request(
        self,
        method,
        url,
        query_params: Any | None = None,
        headers: dict | None = None,
        post_params: dict | None = None,
        body: Any | None = None,
        _preload_content: bool = True,
        _request_timeout: float | tuple[float] | ClientTimeout | None = None,
    ) -> Any: ...
    def parameters_to_tuples(self, params : dict | list, collection_formats: str): ...
    def files_parameters(self, files: Any | None = None) -> list: ...
    def select_header_accept(self, accepts:list[str]) -> str: ...
    def select_header_content_type(
        self,
        content_types: list[str],
        method: str | None = None,
        body: Any | None = None,
    ): ...
    async def update_params_for_auth(
        self, headers: dict, queries: list, auth_settings: list, request_auth: dict | None = None
    ) -> None: ...



from kubernetes_asyncio.client.models import V1ForNode
from kubernetes_asyncio.client.models import V1ForZone
from typing import Any

class V1EndpointHints:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, for_nodes: list[V1ForNode] | None = None, for_zones: list[V1ForZone] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def for_nodes(self) -> list[V1ForNode]: ...
    @for_nodes.setter
    def for_nodes(self, for_nodes: list[V1ForNode]) -> None: ...
    @property
    def for_zones(self) -> list[V1ForZone]: ...
    @for_zones.setter
    def for_zones(self, for_zones: list[V1ForZone]) -> None: ...



from typing import Any

class V1CustomResourceSubresourceScale:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, label_selector_path: str | None = None, spec_replicas_path: str | None = None, status_replicas_path: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def label_selector_path(self) -> str: ...
    @label_selector_path.setter
    def label_selector_path(self, label_selector_path: str) -> None: ...
    @property
    def spec_replicas_path(self) -> str: ...
    @spec_replicas_path.setter
    def spec_replicas_path(self, spec_replicas_path: str) -> None: ...
    @property
    def status_replicas_path(self) -> str: ...
    @status_replicas_path.setter
    def status_replicas_path(self, status_replicas_path: str) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from kubernetes_asyncio.client.models import V1PodFailurePolicy
from kubernetes_asyncio.client.models import V1PodTemplateSpec
from kubernetes_asyncio.client.models import V1SuccessPolicy
from typing import Any

class V1JobSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, active_deadline_seconds: int | None = None, backoff_limit: int | None = None, backoff_limit_per_index: int | None = None, completion_mode: str | None = None, completions: int | None = None, managed_by: str | None = None, manual_selector: bool | None = None, max_failed_indexes: int | None = None, parallelism: int | None = None, pod_failure_policy: V1PodFailurePolicy | None = None, pod_replacement_policy: str | None = None, selector: V1LabelSelector | None = None, success_policy: V1SuccessPolicy | None = None, suspend: bool | None = None, template: V1PodTemplateSpec | None = None, ttl_seconds_after_finished: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def active_deadline_seconds(self) -> int: ...
    @active_deadline_seconds.setter
    def active_deadline_seconds(self, active_deadline_seconds: int) -> None: ...
    @property
    def backoff_limit(self) -> int: ...
    @backoff_limit.setter
    def backoff_limit(self, backoff_limit: int) -> None: ...
    @property
    def backoff_limit_per_index(self) -> int: ...
    @backoff_limit_per_index.setter
    def backoff_limit_per_index(self, backoff_limit_per_index: int) -> None: ...
    @property
    def completion_mode(self) -> str: ...
    @completion_mode.setter
    def completion_mode(self, completion_mode: str) -> None: ...
    @property
    def completions(self) -> int: ...
    @completions.setter
    def completions(self, completions: int) -> None: ...
    @property
    def managed_by(self) -> str: ...
    @managed_by.setter
    def managed_by(self, managed_by: str) -> None: ...
    @property
    def manual_selector(self) -> bool: ...
    @manual_selector.setter
    def manual_selector(self, manual_selector: bool) -> None: ...
    @property
    def max_failed_indexes(self) -> int: ...
    @max_failed_indexes.setter
    def max_failed_indexes(self, max_failed_indexes: int) -> None: ...
    @property
    def parallelism(self) -> int: ...
    @parallelism.setter
    def parallelism(self, parallelism: int) -> None: ...
    @property
    def pod_failure_policy(self) -> V1PodFailurePolicy: ...
    @pod_failure_policy.setter
    def pod_failure_policy(self, pod_failure_policy: V1PodFailurePolicy) -> None: ...
    @property
    def pod_replacement_policy(self) -> str: ...
    @pod_replacement_policy.setter
    def pod_replacement_policy(self, pod_replacement_policy: str) -> None: ...
    @property
    def selector(self) -> V1LabelSelector: ...
    @selector.setter
    def selector(self, selector: V1LabelSelector) -> None: ...
    @property
    def success_policy(self) -> V1SuccessPolicy: ...
    @success_policy.setter
    def success_policy(self, success_policy: V1SuccessPolicy) -> None: ...
    @property
    def suspend(self) -> bool: ...
    @suspend.setter
    def suspend(self, suspend: bool) -> None: ...
    @property
    def template(self) -> V1PodTemplateSpec: ...
    @template.setter
    def template(self, template: V1PodTemplateSpec) -> None: ...
    @property
    def ttl_seconds_after_finished(self) -> int: ...
    @ttl_seconds_after_finished.setter
    def ttl_seconds_after_finished(self, ttl_seconds_after_finished: int) -> None: ...



from kubernetes_asyncio.client.models import V1ServicePort
from kubernetes_asyncio.client.models import V1SessionAffinityConfig
from typing import Any

class V1ServiceSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, allocate_load_balancer_node_ports: bool | None = None, cluster_ip: str | None = None, cluster_ips: list[str] | None = None, external_ips: list[str] | None = None, external_name: str | None = None, external_traffic_policy: str | None = None, health_check_node_port: int | None = None, internal_traffic_policy: str | None = None, ip_families: list[str] | None = None, ip_family_policy: str | None = None, load_balancer_class: str | None = None, load_balancer_ip: str | None = None, load_balancer_source_ranges: list[str] | None = None, ports: list[V1ServicePort] | None = None, publish_not_ready_addresses: bool | None = None, selector: dict[str, str] | None = None, session_affinity: str | None = None, session_affinity_config: V1SessionAffinityConfig | None = None, traffic_distribution: str | None = None, type: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def allocate_load_balancer_node_ports(self) -> bool: ...
    @allocate_load_balancer_node_ports.setter
    def allocate_load_balancer_node_ports(self, allocate_load_balancer_node_ports: bool) -> None: ...
    @property
    def cluster_ip(self) -> str: ...
    @cluster_ip.setter
    def cluster_ip(self, cluster_ip: str) -> None: ...
    @property
    def cluster_ips(self) -> list[str]: ...
    @cluster_ips.setter
    def cluster_ips(self, cluster_ips: list[str]) -> None: ...
    @property
    def external_ips(self) -> list[str]: ...
    @external_ips.setter
    def external_ips(self, external_ips: list[str]) -> None: ...
    @property
    def external_name(self) -> str: ...
    @external_name.setter
    def external_name(self, external_name: str) -> None: ...
    @property
    def external_traffic_policy(self) -> str: ...
    @external_traffic_policy.setter
    def external_traffic_policy(self, external_traffic_policy: str) -> None: ...
    @property
    def health_check_node_port(self) -> int: ...
    @health_check_node_port.setter
    def health_check_node_port(self, health_check_node_port: int) -> None: ...
    @property
    def internal_traffic_policy(self) -> str: ...
    @internal_traffic_policy.setter
    def internal_traffic_policy(self, internal_traffic_policy: str) -> None: ...
    @property
    def ip_families(self) -> list[str]: ...
    @ip_families.setter
    def ip_families(self, ip_families: list[str]) -> None: ...
    @property
    def ip_family_policy(self) -> str: ...
    @ip_family_policy.setter
    def ip_family_policy(self, ip_family_policy: str) -> None: ...
    @property
    def load_balancer_class(self) -> str: ...
    @load_balancer_class.setter
    def load_balancer_class(self, load_balancer_class: str) -> None: ...
    @property
    def load_balancer_ip(self) -> str: ...
    @load_balancer_ip.setter
    def load_balancer_ip(self, load_balancer_ip: str) -> None: ...
    @property
    def load_balancer_source_ranges(self) -> list[str]: ...
    @load_balancer_source_ranges.setter
    def load_balancer_source_ranges(self, load_balancer_source_ranges: list[str]) -> None: ...
    @property
    def ports(self) -> list[V1ServicePort]: ...
    @ports.setter
    def ports(self, ports: list[V1ServicePort]) -> None: ...
    @property
    def publish_not_ready_addresses(self) -> bool: ...
    @publish_not_ready_addresses.setter
    def publish_not_ready_addresses(self, publish_not_ready_addresses: bool) -> None: ...
    @property
    def selector(self) -> dict[str, str]: ...
    @selector.setter
    def selector(self, selector: dict[str, str]) -> None: ...
    @property
    def session_affinity(self) -> str: ...
    @session_affinity.setter
    def session_affinity(self, session_affinity: str) -> None: ...
    @property
    def session_affinity_config(self) -> V1SessionAffinityConfig: ...
    @session_affinity_config.setter
    def session_affinity_config(self, session_affinity_config: V1SessionAffinityConfig) -> None: ...
    @property
    def traffic_distribution(self) -> str: ...
    @traffic_distribution.setter
    def traffic_distribution(self, traffic_distribution: str) -> None: ...
    @property
    def type(self) -> str: ...
    @type.setter
    def type(self, type: str) -> None: ...



from kubernetes_asyncio.client.models import V1NonResourceRule
from kubernetes_asyncio.client.models import V1ResourceRule
from typing import Any

class V1SubjectRulesReviewStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, evaluation_error: str | None = None, incomplete: bool | None = None, non_resource_rules: list[V1NonResourceRule] | None = None, resource_rules: list[V1ResourceRule] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def evaluation_error(self) -> str: ...
    @evaluation_error.setter
    def evaluation_error(self, evaluation_error: str) -> None: ...
    @property
    def incomplete(self) -> bool: ...
    @incomplete.setter
    def incomplete(self, incomplete: bool) -> None: ...
    @property
    def non_resource_rules(self) -> list[V1NonResourceRule]: ...
    @non_resource_rules.setter
    def non_resource_rules(self, non_resource_rules: list[V1NonResourceRule]) -> None: ...
    @property
    def resource_rules(self) -> list[V1ResourceRule]: ...
    @resource_rules.setter
    def resource_rules(self, resource_rules: list[V1ResourceRule]) -> None: ...



from typing import Any

import aiohttp

from kubernetes_asyncio.config.config_exception import ConfigException

GRANT_TYPE_REFRESH_TOKEN = "refresh_token"


class OpenIDRequestor:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        issuer_url: str,
        ssl_ca_cert: Any | None = None,
    ) -> None:
        """OpenIDRequestor implements a very limited subset of the oauth2 APIs that we
        require in order to refresh access tokens"""

        self._client_id = client_id
        self._client_secret = client_secret
        self._issuer_url = issuer_url
        self._ssl_ca_cert = ssl_ca_cert
        self._well_known = None

    def _get_connector(self) -> aiohttp.TCPConnector:
        return aiohttp.TCPConnector(
            verify_ssl=self._ssl_ca_cert is not None, ssl_context=self._ssl_ca_cert
        )

    def _client_session(self) -> aiohttp.ClientSession:
        return aiohttp.ClientSession(
            headers=self._default_headers,
            connector=self._get_connector(),
            auth=aiohttp.BasicAuth(self._client_id, self._client_secret),
            raise_for_status=True,
            trust_env=True,
        )

    async def refresh_token(self, refresh_token: str) -> Any:
        """
        :param refresh_token: an openid refresh-token from a previous token request
        """
        async with self._client_session() as client:
            well_known = await self._get_well_known(client)

            try:
                return await self._post(
                    client,
                    well_known["token_endpoint"],
                    data={
                        "grant_type": GRANT_TYPE_REFRESH_TOKEN,
                        "refresh_token": refresh_token,
                    },
                )
            except aiohttp.ClientResponseError:
                raise ConfigException("oidc: failed to refresh access token")

    async def _get(
        self, client: aiohttp.ClientSession, *args: Any, **kwargs: Any
    ) -> Any:
        async with client.get(*args, **kwargs) as resp:
            return await resp.json()

    async def _post(
        self, client: aiohttp.ClientSession, *args: Any, **kwargs: Any
    ) -> Any:
        async with client.post(*args, **kwargs) as resp:
            return await resp.json()

    async def _get_well_known(self, client: aiohttp.ClientSession) -> Any:
        if self._well_known is None:
            try:
                self._well_known = await self._get(
                    client,
                    "{}/.well-known/openid-configuration".format(
                        self._issuer_url.rstrip("/")
                    ),
                )
            except aiohttp.ClientResponseError:
                raise ConfigException(
                    "oidc: failed to query well-known metadata endpoint"
                )

        return self._well_known

    @property
    def _default_headers(self) -> dict[str, str]:
        return {
            "Accept": "application/json",
            "Content-Type": "application/x-www-form-urlencoded;charset=UTF-8",
        }



from kubernetes_asyncio.client.models import V1PodDNSConfigOption
from typing import Any

class V1PodDNSConfig:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, nameservers: list[str] | None = None, options: list[V1PodDNSConfigOption] | None = None, searches: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def nameservers(self) -> list[str]: ...
    @nameservers.setter
    def nameservers(self, nameservers: list[str]) -> None: ...
    @property
    def options(self) -> list[V1PodDNSConfigOption]: ...
    @options.setter
    def options(self, options: list[V1PodDNSConfigOption]) -> None: ...
    @property
    def searches(self) -> list[str]: ...
    @searches.setter
    def searches(self, searches: list[str]) -> None: ...



from typing import Any

class V1UserInfo:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, extra: dict[str, list[str]] | None = None, groups: list[str] | None = None, uid: str | None = None, username: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def extra(self) -> dict[str, list[str]]: ...
    @extra.setter
    def extra(self, extra: dict[str, list[str]]) -> None: ...
    @property
    def groups(self) -> list[str]: ...
    @groups.setter
    def groups(self, groups: list[str]) -> None: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, uid: str) -> None: ...
    @property
    def username(self) -> str: ...
    @username.setter
    def username(self, username: str) -> None: ...



from kubernetes_asyncio.client.models import V1ContainerRestartRuleOnExitCodes
from typing import Any

class V1ContainerRestartRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, action: str | None = None, exit_codes: V1ContainerRestartRuleOnExitCodes | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def action(self) -> str: ...
    @action.setter
    def action(self, action: str) -> None: ...
    @property
    def exit_codes(self) -> V1ContainerRestartRuleOnExitCodes: ...
    @exit_codes.setter
    def exit_codes(self, exit_codes: V1ContainerRestartRuleOnExitCodes) -> None: ...



from kubernetes_asyncio.client.models import V1ExecAction
from kubernetes_asyncio.client.models import V1HTTPGetAction
from kubernetes_asyncio.client.models import V1SleepAction
from kubernetes_asyncio.client.models import V1TCPSocketAction
from typing import Any

class V1LifecycleHandler:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, _exec: V1ExecAction | None = None, http_get: V1HTTPGetAction | None = None, sleep: V1SleepAction | None = None, tcp_socket: V1TCPSocketAction | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def _exec(self) -> V1ExecAction: ...
    @_exec.setter
    def _exec(self, _exec: V1ExecAction) -> None: ...
    @property
    def http_get(self) -> V1HTTPGetAction: ...
    @http_get.setter
    def http_get(self, http_get: V1HTTPGetAction) -> None: ...
    @property
    def sleep(self) -> V1SleepAction: ...
    @sleep.setter
    def sleep(self, sleep: V1SleepAction) -> None: ...
    @property
    def tcp_socket(self) -> V1TCPSocketAction: ...
    @tcp_socket.setter
    def tcp_socket(self, tcp_socket: V1TCPSocketAction) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class CustomObjectsApi:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_cluster_custom_object(self, group: str, version: str, plural: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def create_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def create_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def create_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def delete_cluster_custom_object(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def delete_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def delete_collection_cluster_custom_object(self, group: str, version: str, plural: str, **kwargs: Any) -> Awaitable[Any]: ...
    def delete_collection_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, **kwargs: Any) -> Awaitable[Any]: ...
    def delete_collection_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def delete_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def delete_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, group: str, version: str, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, group: str, version: str, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def get_cluster_custom_object(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_cluster_custom_object_scale(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_cluster_custom_object_scale_with_http_info(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_cluster_custom_object_status(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_cluster_custom_object_status_with_http_info(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_namespaced_custom_object_scale(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_namespaced_custom_object_scale_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_namespaced_custom_object_status(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[Any]: ...
    def get_namespaced_custom_object_status_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def get_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def list_cluster_custom_object(self, group: str, version: str, plural: str, **kwargs: Any) -> Awaitable[Any]: ...
    def list_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def list_custom_object_for_all_namespaces(self, group: str, version: str, resource_plural: str, **kwargs: Any) -> Awaitable[Any]: ...
    def list_custom_object_for_all_namespaces_with_http_info(self, group: str, version: str, resource_plural: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def list_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, **kwargs: Any) -> Awaitable[Any]: ...
    def list_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_cluster_custom_object(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_cluster_custom_object_scale(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_cluster_custom_object_scale_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_cluster_custom_object_status(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_cluster_custom_object_status_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_namespaced_custom_object_scale(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_namespaced_custom_object_scale_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_namespaced_custom_object_status(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def patch_namespaced_custom_object_status_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def patch_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_cluster_custom_object(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_cluster_custom_object_scale(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_cluster_custom_object_scale_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_cluster_custom_object_status(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_cluster_custom_object_status_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_cluster_custom_object_with_http_info(self, group: str, version: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_namespaced_custom_object(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_namespaced_custom_object_scale(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_namespaced_custom_object_scale_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_namespaced_custom_object_status(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[Any]: ...
    def replace_namespaced_custom_object_status_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...
    def replace_namespaced_custom_object_with_http_info(self, group: str, version: str, namespace: str, plural: str, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[Any, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1CrossVersionObjectReference
from typing import Any

class V1HorizontalPodAutoscalerSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, max_replicas: int | None = None, min_replicas: int | None = None, scale_target_ref: V1CrossVersionObjectReference | None = None, target_cpu_utilization_percentage: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def max_replicas(self) -> int: ...
    @max_replicas.setter
    def max_replicas(self, max_replicas: int) -> None: ...
    @property
    def min_replicas(self) -> int: ...
    @min_replicas.setter
    def min_replicas(self, min_replicas: int) -> None: ...
    @property
    def scale_target_ref(self) -> V1CrossVersionObjectReference: ...
    @scale_target_ref.setter
    def scale_target_ref(self, scale_target_ref: V1CrossVersionObjectReference) -> None: ...
    @property
    def target_cpu_utilization_percentage(self) -> int: ...
    @target_cpu_utilization_percentage.setter
    def target_cpu_utilization_percentage(self, target_cpu_utilization_percentage: int) -> None: ...



from kubernetes_asyncio.client.models import V1PodAffinityTerm
from kubernetes_asyncio.client.models import V1WeightedPodAffinityTerm
from typing import Any

class V1PodAffinity:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, preferred_during_scheduling_ignored_during_execution: list[V1WeightedPodAffinityTerm] | None = None, required_during_scheduling_ignored_during_execution: list[V1PodAffinityTerm] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def preferred_during_scheduling_ignored_during_execution(self) -> list[V1WeightedPodAffinityTerm]: ...
    @preferred_during_scheduling_ignored_during_execution.setter
    def preferred_during_scheduling_ignored_during_execution(self, preferred_during_scheduling_ignored_during_execution: list[V1WeightedPodAffinityTerm]) -> None: ...
    @property
    def required_during_scheduling_ignored_during_execution(self) -> list[V1PodAffinityTerm]: ...
    @required_during_scheduling_ignored_during_execution.setter
    def required_during_scheduling_ignored_during_execution(self, required_during_scheduling_ignored_during_execution: list[V1PodAffinityTerm]) -> None: ...



from kubernetes_asyncio.client.models import V1NetworkPolicyPeer
from kubernetes_asyncio.client.models import V1NetworkPolicyPort
from typing import Any

class V1NetworkPolicyEgressRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ports: list[V1NetworkPolicyPort] | None = None, to: list[V1NetworkPolicyPeer] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def ports(self) -> list[V1NetworkPolicyPort]: ...
    @ports.setter
    def ports(self, ports: list[V1NetworkPolicyPort]) -> None: ...
    @property
    def to(self) -> list[V1NetworkPolicyPeer]: ...
    @to.setter
    def to(self, to: list[V1NetworkPolicyPeer]) -> None: ...



from kubernetes_asyncio.client.models import V1alpha3DeviceSelector
from typing import Any

class V1alpha3DeviceTaintSelector:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, device: str | None = None, device_class_name: str | None = None, driver: str | None = None, pool: str | None = None, selectors: list[V1alpha3DeviceSelector] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def device(self) -> str: ...
    @device.setter
    def device(self, device: str) -> None: ...
    @property
    def device_class_name(self) -> str: ...
    @device_class_name.setter
    def device_class_name(self, device_class_name: str) -> None: ...
    @property
    def driver(self) -> str: ...
    @driver.setter
    def driver(self, driver: str) -> None: ...
    @property
    def pool(self) -> str: ...
    @pool.setter
    def pool(self, pool: str) -> None: ...
    @property
    def selectors(self) -> list[V1alpha3DeviceSelector]: ...
    @selectors.setter
    def selectors(self, selectors: list[V1alpha3DeviceSelector]) -> None: ...



from kubernetes_asyncio.client.models import V1beta1CapacityRequirements
from kubernetes_asyncio.client.models import V1beta1DeviceSelector
from kubernetes_asyncio.client.models import V1beta1DeviceSubRequest
from kubernetes_asyncio.client.models import V1beta1DeviceToleration
from typing import Any

class V1beta1DeviceRequest:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, admin_access: bool | None = None, allocation_mode: str | None = None, capacity: V1beta1CapacityRequirements | None = None, count: int | None = None, device_class_name: str | None = None, first_available: list[V1beta1DeviceSubRequest] | None = None, name: str | None = None, selectors: list[V1beta1DeviceSelector] | None = None, tolerations: list[V1beta1DeviceToleration] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def admin_access(self) -> bool: ...
    @admin_access.setter
    def admin_access(self, admin_access: bool) -> None: ...
    @property
    def allocation_mode(self) -> str: ...
    @allocation_mode.setter
    def allocation_mode(self, allocation_mode: str) -> None: ...
    @property
    def capacity(self) -> V1beta1CapacityRequirements: ...
    @capacity.setter
    def capacity(self, capacity: V1beta1CapacityRequirements) -> None: ...
    @property
    def count(self) -> int: ...
    @count.setter
    def count(self, count: int) -> None: ...
    @property
    def device_class_name(self) -> str: ...
    @device_class_name.setter
    def device_class_name(self, device_class_name: str) -> None: ...
    @property
    def first_available(self) -> list[V1beta1DeviceSubRequest]: ...
    @first_available.setter
    def first_available(self, first_available: list[V1beta1DeviceSubRequest]) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def selectors(self) -> list[V1beta1DeviceSelector]: ...
    @selectors.setter
    def selectors(self, selectors: list[V1beta1DeviceSelector]) -> None: ...
    @property
    def tolerations(self) -> list[V1beta1DeviceToleration]: ...
    @tolerations.setter
    def tolerations(self, tolerations: list[V1beta1DeviceToleration]) -> None: ...



from typing import Any

class V1AzureFilePersistentVolumeSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, read_only: bool | None = None, secret_name: str | None = None, secret_namespace: str | None = None, share_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def read_only(self) -> bool: ...
    @read_only.setter
    def read_only(self, read_only: bool) -> None: ...
    @property
    def secret_name(self) -> str: ...
    @secret_name.setter
    def secret_name(self, secret_name: str) -> None: ...
    @property
    def secret_namespace(self) -> str: ...
    @secret_namespace.setter
    def secret_namespace(self, secret_namespace: str) -> None: ...
    @property
    def share_name(self) -> str: ...
    @share_name.setter
    def share_name(self, share_name: str) -> None: ...



# coding: utf-8

"""
    Kubernetes

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: v1.34.3
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from kubernetes_asyncio.client.configuration import Configuration


class V1beta2DeviceToleration(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'effect': 'str',
        'key': 'str',
        'operator': 'str',
        'toleration_seconds': 'int',
        'value': 'str'
    }

    attribute_map = {
        'effect': 'effect',
        'key': 'key',
        'operator': 'operator',
        'toleration_seconds': 'tolerationSeconds',
        'value': 'value'
    }

    def __init__(self, effect=None, key=None, operator=None, toleration_seconds=None, value=None, local_vars_configuration=None):  # noqa: E501
        """V1beta2DeviceToleration - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default()
        self.local_vars_configuration = local_vars_configuration

        self._effect = None
        self._key = None
        self._operator = None
        self._toleration_seconds = None
        self._value = None
        self.discriminator = None

        if effect is not None:
            self.effect = effect
        if key is not None:
            self.key = key
        if operator is not None:
            self.operator = operator
        if toleration_seconds is not None:
            self.toleration_seconds = toleration_seconds
        if value is not None:
            self.value = value

    @property
    def effect(self):
        """Gets the effect of this V1beta2DeviceToleration.  # noqa: E501

        Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.  # noqa: E501

        :return: The effect of this V1beta2DeviceToleration.  # noqa: E501
        :rtype: str
        """
        return self._effect

    @effect.setter
    def effect(self, effect):
        """Sets the effect of this V1beta2DeviceToleration.

        Effect indicates the taint effect to match. Empty means match all taint effects. When specified, allowed values are NoSchedule and NoExecute.  # noqa: E501

        :param effect: The effect of this V1beta2DeviceToleration.  # noqa: E501
        :type effect: str
        """

        self._effect = effect

    @property
    def key(self):
        """Gets the key of this V1beta2DeviceToleration.  # noqa: E501

        Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.  # noqa: E501

        :return: The key of this V1beta2DeviceToleration.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this V1beta2DeviceToleration.

        Key is the taint key that the toleration applies to. Empty means match all taint keys. If the key is empty, operator must be Exists; this combination means to match all values and all keys. Must be a label name.  # noqa: E501

        :param key: The key of this V1beta2DeviceToleration.  # noqa: E501
        :type key: str
        """

        self._key = key

    @property
    def operator(self):
        """Gets the operator of this V1beta2DeviceToleration.  # noqa: E501

        Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.  # noqa: E501

        :return: The operator of this V1beta2DeviceToleration.  # noqa: E501
        :rtype: str
        """
        return self._operator

    @operator.setter
    def operator(self, operator):
        """Sets the operator of this V1beta2DeviceToleration.

        Operator represents a key's relationship to the value. Valid operators are Exists and Equal. Defaults to Equal. Exists is equivalent to wildcard for value, so that a ResourceClaim can tolerate all taints of a particular category.  # noqa: E501

        :param operator: The operator of this V1beta2DeviceToleration.  # noqa: E501
        :type operator: str
        """

        self._operator = operator

    @property
    def toleration_seconds(self):
        """Gets the toleration_seconds of this V1beta2DeviceToleration.  # noqa: E501

        TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.  # noqa: E501

        :return: The toleration_seconds of this V1beta2DeviceToleration.  # noqa: E501
        :rtype: int
        """
        return self._toleration_seconds

    @toleration_seconds.setter
    def toleration_seconds(self, toleration_seconds):
        """Sets the toleration_seconds of this V1beta2DeviceToleration.

        TolerationSeconds represents the period of time the toleration (which must be of effect NoExecute, otherwise this field is ignored) tolerates the taint. By default, it is not set, which means tolerate the taint forever (do not evict). Zero and negative values will be treated as 0 (evict immediately) by the system. If larger than zero, the time when the pod needs to be evicted is calculated as <time when taint was adedd> + <toleration seconds>.  # noqa: E501

        :param toleration_seconds: The toleration_seconds of this V1beta2DeviceToleration.  # noqa: E501
        :type toleration_seconds: int
        """

        self._toleration_seconds = toleration_seconds

    @property
    def value(self):
        """Gets the value of this V1beta2DeviceToleration.  # noqa: E501

        Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.  # noqa: E501

        :return: The value of this V1beta2DeviceToleration.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this V1beta2DeviceToleration.

        Value is the taint value the toleration matches to. If the operator is Exists, the value must be empty, otherwise just a regular string. Must be a label value.  # noqa: E501

        :param value: The value of this V1beta2DeviceToleration.  # noqa: E501
        :type value: str
        """

        self._value = value

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta2DeviceToleration):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta2DeviceToleration):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2021 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import asyncio
import json
import unittest
from typing import Callable
from unittest import IsolatedAsyncioTestCase

from kubernetes_asyncio.client.rest import ApiException
from kubernetes_asyncio.leaderelection import electionconfig, leaderelection
from kubernetes_asyncio.leaderelection.leaderelectionrecord import (
    LeaderElectionRecord,
)
from kubernetes_asyncio.leaderelection.resourcelock.baselock import BaseLock


class LeaderElectionTest(IsolatedAsyncioTestCase):
    async def test_simple_leader_election(self) -> None:
        election_history = []
        leadership_history = []

        def on_create():
            election_history.append("create record")
            leadership_history.append("get leadership")

        def on_update():
            election_history.append("update record")

        def on_change():
            election_history.append("change record")

        mock_lock = MockResourceLock(
            "mock",
            "mock_namespace",
            "mock",
            asyncio.Lock(),
            on_create,
            on_update,
            on_change,
            None,
        )

        async def on_started_leading() -> None:
            leadership_history.append("start leading")

        async def on_stopped_leading() -> None:
            leadership_history.append("stop leading")

        # Create config 4.5 4 3
        config = electionconfig.Config(
            lock=mock_lock,
            lease_duration=2.5,
            renew_deadline=2,
            retry_period=1.5,
            onstarted_leading=on_started_leading(),
            onstopped_leading=on_stopped_leading(),
        )

        # Enter leader election
        await leaderelection.LeaderElection(config).run()

        self.assert_history(
            election_history,
            ["create record", "update record", "update record", "update record"],
        )
        self.assert_history(
            leadership_history, ["get leadership", "start leading", "stop leading"]
        )

    async def test_leader_election(self) -> None:
        election_history = []
        leadership_history = []

        def on_create_A():
            election_history.append("A creates record")
            leadership_history.append("A gets leadership")

        def on_update_A():
            election_history.append("A updates record")

        def on_change_A():
            election_history.append("A gets leadership")

        lock = asyncio.Lock()

        mock_lock_A = MockResourceLock(
            "mock",
            "mock_namespace",
            "MockA",
            lock,
            on_create_A,
            on_update_A,
            on_change_A,
            None,
        )
        mock_lock_A.renew_count_max = 3

        async def on_started_leading_A():
            leadership_history.append("A starts leading")

        async def on_stopped_leading_A():
            leadership_history.append("A stops leading")

        config_A = electionconfig.Config(
            lock=mock_lock_A,
            lease_duration=2.5,
            renew_deadline=2,
            retry_period=1.5,
            onstarted_leading=on_started_leading_A(),
            onstopped_leading=on_stopped_leading_A(),
        )

        def on_create_B():
            election_history.append("B creates record")
            leadership_history.append("B gets leadership")

        def on_update_B():
            election_history.append("B updates record")

        def on_change_B():
            leadership_history.append("B gets leadership")

        mock_lock_B = MockResourceLock(
            "mock",
            "mock_namespace",
            "MockB",
            lock,
            on_create_B,
            on_update_B,
            on_change_B,
            None,
        )
        mock_lock_B.renew_count_max = 4

        async def on_started_leading_B():
            leadership_history.append("B starts leading")

        async def on_stopped_leading_B():
            leadership_history.append("B stops leading")

        config_B = electionconfig.Config(
            lock=mock_lock_B,
            lease_duration=2.5,
            renew_deadline=2,
            retry_period=1.5,
            onstarted_leading=on_started_leading_B(),
            onstopped_leading=on_stopped_leading_B(),
        )

        mock_lock_B.leader_record = mock_lock_A.leader_record

        config_A_election = asyncio.create_task(
            leaderelection.LeaderElection(config_A).run()
        )
        config_B_election = asyncio.create_task(
            leaderelection.LeaderElection(config_B).run()
        )

        await asyncio.gather(config_A_election, config_B_election)

        self.assert_history(
            election_history,
            [
                "A creates record",
                "A updates record",
                "A updates record",
                "B updates record",
                "B updates record",
                "B updates record",
                "B updates record",
            ],
        )
        self.assert_history(
            leadership_history,
            [
                "A gets leadership",
                "A starts leading",
                "A stops leading",
                "B gets leadership",
                "B starts leading",
                "B stops leading",
            ],
        )

    """Expected behavior: to check if the leader stops leading if it fails to update the lock within the renew_deadline
    and stops leading after finally timing out. The difference between each try comes out to be approximately the sleep
    time.
    Example:
    create record:  0s
    on try update:  1.5s
    on update:  zzz s
    on try update:  3s
    on update: zzz s
    on try update:  4.5s
    on try update:  6s
    Timeout - Leader Exits"""

    async def test_leader_election_with_renew_deadline(self) -> None:
        election_history = []
        leadership_history = []

        def on_create():
            election_history.append("create record")
            leadership_history.append("get leadership")

        def on_update():
            election_history.append("update record")

        def on_change():
            election_history.append("change record")

        def on_try_update():
            election_history.append("try update record")

        mock_lock = MockResourceLock(
            "mock",
            "mock_namespace",
            "mock",
            asyncio.Lock(),
            on_create,
            on_update,
            on_change,
            on_try_update,
        )
        mock_lock.renew_count_max = 3

        async def on_started_leading():
            leadership_history.append("start leading")

        async def on_stopped_leading():
            leadership_history.append("stop leading")

        # Create config
        config = electionconfig.Config(
            lock=mock_lock,
            lease_duration=2.5,
            renew_deadline=2,
            retry_period=1.5,
            onstarted_leading=on_started_leading(),
            onstopped_leading=on_stopped_leading(),
        )

        # Enter leader election
        await leaderelection.LeaderElection(config).run()

        self.assert_history(
            election_history,
            [
                "create record",
                "try update record",
                "update record",
                "try update record",
                "update record",
                "try update record",
                "try update record",
            ],
        )

        self.assert_history(
            leadership_history, ["get leadership", "start leading", "stop leading"]
        )

    def assert_history(self, history, expected) -> None:
        self.assertIsNotNone(expected)
        self.assertIsNotNone(history)
        self.assertEqual(len(expected), len(history))

        for idx in range(len(history)):
            self.assertEqual(
                history[idx],
                expected[idx],
                msg="Not equal at index {}, expected {}, got {}".format(
                    idx, expected[idx], history[idx]
                ),
            )


class MockResourceLock(BaseLock):
    def __init__(
        self,
        name: str,
        namespace: str,
        identity: str,
        shared_lock: asyncio.Lock,
        on_create: Callable,
        on_update: Callable,
        on_change: Callable,
        on_try_update: Callable | None,
    ) -> None:
        # self.leader_record is shared between two MockResourceLock objects
        self.leader_record: list[LeaderElectionRecord] = []
        self.renew_count = 0
        self.renew_count_max = 4
        self.name = name
        self.namespace = namespace
        self.identity = str(identity)
        self.lock = shared_lock

        self.on_create = on_create
        self.on_update = on_update
        self.on_change = on_change
        self.on_try_update = on_try_update

    async def get(
        self, name: str, namespace: str
    ) -> tuple[bool, LeaderElectionRecord] | tuple[bool, Exception] | tuple[bool, None]:
        await self.lock.acquire()
        try:
            if self.leader_record:
                return True, self.leader_record[0]

            ex = ApiException()
            ex.body = json.dumps({"code": 404}).encode()
            return False, ex
        finally:
            self.lock.release()

    async def create(
        self, name: str, namespace: str, election_record: LeaderElectionRecord
    ) -> bool:
        await self.lock.acquire()
        try:
            if len(self.leader_record) == 1:
                return False
            self.leader_record.append(election_record)
            self.on_create()
            self.renew_count += 1
            return True
        finally:
            self.lock.release()

    async def update(
        self, name: str, namespace: str, updated_record: LeaderElectionRecord
    ) -> bool:
        await self.lock.acquire()
        try:
            if self.on_try_update:
                self.on_try_update()
            if self.renew_count >= self.renew_count_max:
                return False

            old_record = self.leader_record[0]
            self.leader_record[0] = updated_record

            self.on_update()

            if old_record.holder_identity != updated_record.holder_identity:
                await asyncio.sleep(2)
                self.on_change()

            self.renew_count += 1
            return True
        finally:
            self.lock.release()


if __name__ == "__main__":
    unittest.main()



from kubernetes_asyncio.client.models import V1CounterSet
from kubernetes_asyncio.client.models import V1Device
from kubernetes_asyncio.client.models import V1NodeSelector
from kubernetes_asyncio.client.models import V1ResourcePool
from typing import Any

class V1ResourceSliceSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, all_nodes: bool | None = None, devices: list[V1Device] | None = None, driver: str | None = None, node_name: str | None = None, node_selector: V1NodeSelector | None = None, per_device_node_selection: bool | None = None, pool: V1ResourcePool | None = None, shared_counters: list[V1CounterSet] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def all_nodes(self) -> bool: ...
    @all_nodes.setter
    def all_nodes(self, all_nodes: bool) -> None: ...
    @property
    def devices(self) -> list[V1Device]: ...
    @devices.setter
    def devices(self, devices: list[V1Device]) -> None: ...
    @property
    def driver(self) -> str: ...
    @driver.setter
    def driver(self, driver: str) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def node_selector(self) -> V1NodeSelector: ...
    @node_selector.setter
    def node_selector(self, node_selector: V1NodeSelector) -> None: ...
    @property
    def per_device_node_selection(self) -> bool: ...
    @per_device_node_selection.setter
    def per_device_node_selection(self, per_device_node_selection: bool) -> None: ...
    @property
    def pool(self) -> V1ResourcePool: ...
    @pool.setter
    def pool(self, pool: V1ResourcePool) -> None: ...
    @property
    def shared_counters(self) -> list[V1CounterSet]: ...
    @shared_counters.setter
    def shared_counters(self, shared_counters: list[V1CounterSet]) -> None: ...



from kubernetes_asyncio.client.models import V1DaemonSetCondition
from typing import Any

class V1DaemonSetStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, collision_count: int | None = None, conditions: list[V1DaemonSetCondition] | None = None, current_number_scheduled: int | None = None, desired_number_scheduled: int | None = None, number_available: int | None = None, number_misscheduled: int | None = None, number_ready: int | None = None, number_unavailable: int | None = None, observed_generation: int | None = None, updated_number_scheduled: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def collision_count(self) -> int: ...
    @collision_count.setter
    def collision_count(self, collision_count: int) -> None: ...
    @property
    def conditions(self) -> list[V1DaemonSetCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1DaemonSetCondition]) -> None: ...
    @property
    def current_number_scheduled(self) -> int: ...
    @current_number_scheduled.setter
    def current_number_scheduled(self, current_number_scheduled: int) -> None: ...
    @property
    def desired_number_scheduled(self) -> int: ...
    @desired_number_scheduled.setter
    def desired_number_scheduled(self, desired_number_scheduled: int) -> None: ...
    @property
    def number_available(self) -> int: ...
    @number_available.setter
    def number_available(self, number_available: int) -> None: ...
    @property
    def number_misscheduled(self) -> int: ...
    @number_misscheduled.setter
    def number_misscheduled(self, number_misscheduled: int) -> None: ...
    @property
    def number_ready(self) -> int: ...
    @number_ready.setter
    def number_ready(self, number_ready: int) -> None: ...
    @property
    def number_unavailable(self) -> int: ...
    @number_unavailable.setter
    def number_unavailable(self, number_unavailable: int) -> None: ...
    @property
    def observed_generation(self) -> int: ...
    @observed_generation.setter
    def observed_generation(self, observed_generation: int) -> None: ...
    @property
    def updated_number_scheduled(self) -> int: ...
    @updated_number_scheduled.setter
    def updated_number_scheduled(self, updated_number_scheduled: int) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1beta1VolumeAttributesClass
from kubernetes_asyncio.client.models import V1beta1VolumeAttributesClassList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class StorageV1beta1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_volume_attributes_class(self, body: V1beta1VolumeAttributesClass, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClass]: ...
    def create_volume_attributes_class_with_http_info(self, body: V1beta1VolumeAttributesClass, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def delete_collection_volume_attributes_class(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_volume_attributes_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_volume_attributes_class(self, name: str, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClass]: ...
    def delete_volume_attributes_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_volume_attributes_class(self, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClassList]: ...
    def list_volume_attributes_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClassList, int, CIMultiDictProxy]]: ...
    def patch_volume_attributes_class(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClass]: ...
    def patch_volume_attributes_class_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def read_volume_attributes_class(self, name: str, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClass]: ...
    def read_volume_attributes_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def replace_volume_attributes_class(self, name: str, body: V1beta1VolumeAttributesClass, **kwargs: Any) -> Awaitable[V1beta1VolumeAttributesClass]: ...
    def replace_volume_attributes_class_with_http_info(self, name: str, body: V1beta1VolumeAttributesClass, **kwargs: Any) -> Awaitable[tuple[V1beta1VolumeAttributesClass, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1StatusCause
from typing import Any

class V1StatusDetails:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, causes: list[V1StatusCause] | None = None, group: str | None = None, kind: str | None = None, name: str | None = None, retry_after_seconds: int | None = None, uid: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def causes(self) -> list[V1StatusCause]: ...
    @causes.setter
    def causes(self, causes: list[V1StatusCause]) -> None: ...
    @property
    def group(self) -> str: ...
    @group.setter
    def group(self, group: str) -> None: ...
    @property
    def kind(self) -> str: ...
    @kind.setter
    def kind(self, kind: str) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def retry_after_seconds(self) -> int: ...
    @retry_after_seconds.setter
    def retry_after_seconds(self, retry_after_seconds: int) -> None: ...
    @property
    def uid(self) -> str: ...
    @uid.setter
    def uid(self, uid: str) -> None: ...



from kubernetes_asyncio.client.models import V1BoundObjectReference
from typing import Any

class V1TokenRequestSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, audiences: list[str] | None = None, bound_object_ref: V1BoundObjectReference | None = None, expiration_seconds: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def audiences(self) -> list[str]: ...
    @audiences.setter
    def audiences(self, audiences: list[str]) -> None: ...
    @property
    def bound_object_ref(self) -> V1BoundObjectReference: ...
    @bound_object_ref.setter
    def bound_object_ref(self, bound_object_ref: V1BoundObjectReference) -> None: ...
    @property
    def expiration_seconds(self) -> int: ...
    @expiration_seconds.setter
    def expiration_seconds(self, expiration_seconds: int) -> None: ...



import asyncio.subprocess
import json
import shlex
from types import SimpleNamespace
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from kubernetes_asyncio.config.kube_config import ConfigNode


async def google_auth_credentials(provider: "ConfigNode") -> SimpleNamespace:
    if "cmd-path" not in provider or "cmd-args" not in provider:
        raise ValueError(
            "GoogleAuth via gcloud is supported! Values for cmd-path, cmd-args are required."
        )
    cmd_args = provider["cmd-args"]
    cmd_path = provider["cmd-path"]
    if not isinstance(cmd_args, str) or not isinstance(cmd_path, str):
        raise ValueError(
            "GoogleAuth via gcloud is supported! Values for cmd-path, cmd-args have to be strings."
        )

    cmd_args_splited = shlex.split(cmd_args)
    cmd_exec = asyncio.create_subprocess_exec(
        cmd_path,
        *cmd_args_splited,
        stdin=None,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
    )
    proc = await cmd_exec

    if proc.stdout:
        raw_data = await proc.stdout.read()
    else:
        raise RuntimeError("Unable to read stdout")
    data = json.loads(raw_data.decode("ascii").rstrip())

    await proc.wait()
    return SimpleNamespace(
        token=data["credential"]["access_token"],
        expiry=data["credential"]["token_expiry"],
    )



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1IPAddress
from kubernetes_asyncio.client.models import V1IPAddressList
from kubernetes_asyncio.client.models import V1Ingress
from kubernetes_asyncio.client.models import V1IngressClass
from kubernetes_asyncio.client.models import V1IngressClassList
from kubernetes_asyncio.client.models import V1IngressList
from kubernetes_asyncio.client.models import V1NetworkPolicy
from kubernetes_asyncio.client.models import V1NetworkPolicyList
from kubernetes_asyncio.client.models import V1ServiceCIDR
from kubernetes_asyncio.client.models import V1ServiceCIDRList
from kubernetes_asyncio.client.models import V1Status
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class NetworkingV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_ingress_class(self, body: V1IngressClass, **kwargs: Any) -> Awaitable[V1IngressClass]: ...
    def create_ingress_class_with_http_info(self, body: V1IngressClass, **kwargs: Any) -> Awaitable[tuple[V1IngressClass, int, CIMultiDictProxy]]: ...
    def create_ip_address(self, body: V1IPAddress, **kwargs: Any) -> Awaitable[V1IPAddress]: ...
    def create_ip_address_with_http_info(self, body: V1IPAddress, **kwargs: Any) -> Awaitable[tuple[V1IPAddress, int, CIMultiDictProxy]]: ...
    def create_namespaced_ingress(self, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def create_namespaced_ingress_with_http_info(self, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def create_namespaced_network_policy(self, namespace: str, body: V1NetworkPolicy, **kwargs: Any) -> Awaitable[V1NetworkPolicy]: ...
    def create_namespaced_network_policy_with_http_info(self, namespace: str, body: V1NetworkPolicy, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicy, int, CIMultiDictProxy]]: ...
    def create_service_cidr(self, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def create_service_cidr_with_http_info(self, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def delete_collection_ingress_class(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_ingress_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_ip_address(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_ip_address_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_ingress(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_ingress_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_network_policy(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_network_policy_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_service_cidr(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_service_cidr_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_ingress_class(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_ingress_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_ip_address(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_ip_address_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_ingress(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_ingress_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_namespaced_network_policy(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_network_policy_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_service_cidr(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_service_cidr_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_ingress_class(self, **kwargs: Any) -> Awaitable[V1IngressClassList]: ...
    def list_ingress_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1IngressClassList, int, CIMultiDictProxy]]: ...
    def list_ingress_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1IngressList]: ...
    def list_ingress_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1IngressList, int, CIMultiDictProxy]]: ...
    def list_ip_address(self, **kwargs: Any) -> Awaitable[V1IPAddressList]: ...
    def list_ip_address_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1IPAddressList, int, CIMultiDictProxy]]: ...
    def list_namespaced_ingress(self, namespace: str, **kwargs: Any) -> Awaitable[V1IngressList]: ...
    def list_namespaced_ingress_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1IngressList, int, CIMultiDictProxy]]: ...
    def list_namespaced_network_policy(self, namespace: str, **kwargs: Any) -> Awaitable[V1NetworkPolicyList]: ...
    def list_namespaced_network_policy_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicyList, int, CIMultiDictProxy]]: ...
    def list_network_policy_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1NetworkPolicyList]: ...
    def list_network_policy_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicyList, int, CIMultiDictProxy]]: ...
    def list_service_cidr(self, **kwargs: Any) -> Awaitable[V1ServiceCIDRList]: ...
    def list_service_cidr_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDRList, int, CIMultiDictProxy]]: ...
    def patch_ingress_class(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1IngressClass]: ...
    def patch_ingress_class_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1IngressClass, int, CIMultiDictProxy]]: ...
    def patch_ip_address(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1IPAddress]: ...
    def patch_ip_address_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1IPAddress, int, CIMultiDictProxy]]: ...
    def patch_namespaced_ingress(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def patch_namespaced_ingress_status(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def patch_namespaced_ingress_status_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def patch_namespaced_ingress_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def patch_namespaced_network_policy(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1NetworkPolicy]: ...
    def patch_namespaced_network_policy_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicy, int, CIMultiDictProxy]]: ...
    def patch_service_cidr(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def patch_service_cidr_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def patch_service_cidr_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def patch_service_cidr_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def read_ingress_class(self, name: str, **kwargs: Any) -> Awaitable[V1IngressClass]: ...
    def read_ingress_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1IngressClass, int, CIMultiDictProxy]]: ...
    def read_ip_address(self, name: str, **kwargs: Any) -> Awaitable[V1IPAddress]: ...
    def read_ip_address_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1IPAddress, int, CIMultiDictProxy]]: ...
    def read_namespaced_ingress(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def read_namespaced_ingress_status(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def read_namespaced_ingress_status_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def read_namespaced_ingress_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def read_namespaced_network_policy(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1NetworkPolicy]: ...
    def read_namespaced_network_policy_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicy, int, CIMultiDictProxy]]: ...
    def read_service_cidr(self, name: str, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def read_service_cidr_status(self, name: str, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def read_service_cidr_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def read_service_cidr_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def replace_ingress_class(self, name: str, body: V1IngressClass, **kwargs: Any) -> Awaitable[V1IngressClass]: ...
    def replace_ingress_class_with_http_info(self, name: str, body: V1IngressClass, **kwargs: Any) -> Awaitable[tuple[V1IngressClass, int, CIMultiDictProxy]]: ...
    def replace_ip_address(self, name: str, body: V1IPAddress, **kwargs: Any) -> Awaitable[V1IPAddress]: ...
    def replace_ip_address_with_http_info(self, name: str, body: V1IPAddress, **kwargs: Any) -> Awaitable[tuple[V1IPAddress, int, CIMultiDictProxy]]: ...
    def replace_namespaced_ingress(self, name: str, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def replace_namespaced_ingress_status(self, name: str, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[V1Ingress]: ...
    def replace_namespaced_ingress_status_with_http_info(self, name: str, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def replace_namespaced_ingress_with_http_info(self, name: str, namespace: str, body: V1Ingress, **kwargs: Any) -> Awaitable[tuple[V1Ingress, int, CIMultiDictProxy]]: ...
    def replace_namespaced_network_policy(self, name: str, namespace: str, body: V1NetworkPolicy, **kwargs: Any) -> Awaitable[V1NetworkPolicy]: ...
    def replace_namespaced_network_policy_with_http_info(self, name: str, namespace: str, body: V1NetworkPolicy, **kwargs: Any) -> Awaitable[tuple[V1NetworkPolicy, int, CIMultiDictProxy]]: ...
    def replace_service_cidr(self, name: str, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def replace_service_cidr_status(self, name: str, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[V1ServiceCIDR]: ...
    def replace_service_cidr_status_with_http_info(self, name: str, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...
    def replace_service_cidr_with_http_info(self, name: str, body: V1ServiceCIDR, **kwargs: Any) -> Awaitable[tuple[V1ServiceCIDR, int, CIMultiDictProxy]]: ...



from typing import Any

class V1SleepAction:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, seconds: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def seconds(self) -> int: ...
    @seconds.setter
    def seconds(self, seconds: int) -> None: ...



from kubernetes_asyncio.client.models import V1alpha3CELDeviceSelector
from typing import Any

class V1alpha3DeviceSelector:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, cel: V1alpha3CELDeviceSelector | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def cel(self) -> V1alpha3CELDeviceSelector: ...
    @cel.setter
    def cel(self, cel: V1alpha3CELDeviceSelector) -> None: ...



from kubernetes_asyncio.client.models import V1WebhookConversion
from typing import Any

class V1CustomResourceConversion:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, strategy: str | None = None, webhook: V1WebhookConversion | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def strategy(self) -> str: ...
    @strategy.setter
    def strategy(self, strategy: str) -> None: ...
    @property
    def webhook(self) -> V1WebhookConversion: ...
    @webhook.setter
    def webhook(self, webhook: V1WebhookConversion) -> None: ...



from kubernetes_asyncio.client.models import V1beta1AllocatedDeviceStatus
from kubernetes_asyncio.client.models import V1beta1AllocationResult
from kubernetes_asyncio.client.models import V1beta1ResourceClaimConsumerReference
from typing import Any

class V1beta1ResourceClaimStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, allocation: V1beta1AllocationResult | None = None, devices: list[V1beta1AllocatedDeviceStatus] | None = None, reserved_for: list[V1beta1ResourceClaimConsumerReference] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def allocation(self) -> V1beta1AllocationResult: ...
    @allocation.setter
    def allocation(self, allocation: V1beta1AllocationResult) -> None: ...
    @property
    def devices(self) -> list[V1beta1AllocatedDeviceStatus]: ...
    @devices.setter
    def devices(self, devices: list[V1beta1AllocatedDeviceStatus]) -> None: ...
    @property
    def reserved_for(self) -> list[V1beta1ResourceClaimConsumerReference]: ...
    @reserved_for.setter
    def reserved_for(self, reserved_for: list[V1beta1ResourceClaimConsumerReference]) -> None: ...



from kubernetes_asyncio.client.models import V1FieldSelectorRequirement
from typing import Any

class V1FieldSelectorAttributes:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, raw_selector: str | None = None, requirements: list[V1FieldSelectorRequirement] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def raw_selector(self) -> str: ...
    @raw_selector.setter
    def raw_selector(self, raw_selector: str) -> None: ...
    @property
    def requirements(self) -> list[V1FieldSelectorRequirement]: ...
    @requirements.setter
    def requirements(self, requirements: list[V1FieldSelectorRequirement]) -> None: ...



from kubernetes_asyncio.client.models import V1LabelSelector
from typing import Any

class V1PodAffinityTerm:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, label_selector: V1LabelSelector | None = None, match_label_keys: list[str] | None = None, mismatch_label_keys: list[str] | None = None, namespace_selector: V1LabelSelector | None = None, namespaces: list[str] | None = None, topology_key: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def label_selector(self) -> V1LabelSelector: ...
    @label_selector.setter
    def label_selector(self, label_selector: V1LabelSelector) -> None: ...
    @property
    def match_label_keys(self) -> list[str]: ...
    @match_label_keys.setter
    def match_label_keys(self, match_label_keys: list[str]) -> None: ...
    @property
    def mismatch_label_keys(self) -> list[str]: ...
    @mismatch_label_keys.setter
    def mismatch_label_keys(self, mismatch_label_keys: list[str]) -> None: ...
    @property
    def namespace_selector(self) -> V1LabelSelector: ...
    @namespace_selector.setter
    def namespace_selector(self, namespace_selector: V1LabelSelector) -> None: ...
    @property
    def namespaces(self) -> list[str]: ...
    @namespaces.setter
    def namespaces(self, namespaces: list[str]) -> None: ...
    @property
    def topology_key(self) -> str: ...
    @topology_key.setter
    def topology_key(self, topology_key: str) -> None: ...



from typing import Any

class V1NonResourceRule:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, non_resource_urls: list[str] | None = None, verbs: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def non_resource_urls(self) -> list[str]: ...
    @non_resource_urls.setter
    def non_resource_urls(self, non_resource_urls: list[str]) -> None: ...
    @property
    def verbs(self) -> list[str]: ...
    @verbs.setter
    def verbs(self, verbs: list[str]) -> None: ...



from datetime import datetime
from typing import Any

class V1ContainerStateTerminated:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, container_id: str | None = None, exit_code: int | None = None, finished_at: datetime | None = None, message: str | None = None, reason: str | None = None, signal: int | None = None, started_at: datetime | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def container_id(self) -> str: ...
    @container_id.setter
    def container_id(self, container_id: str) -> None: ...
    @property
    def exit_code(self) -> int: ...
    @exit_code.setter
    def exit_code(self, exit_code: int) -> None: ...
    @property
    def finished_at(self) -> datetime: ...
    @finished_at.setter
    def finished_at(self, finished_at: datetime) -> None: ...
    @property
    def message(self) -> str: ...
    @message.setter
    def message(self, message: str) -> None: ...
    @property
    def reason(self) -> str: ...
    @reason.setter
    def reason(self, reason: str) -> None: ...
    @property
    def signal(self) -> int: ...
    @signal.setter
    def signal(self, signal: int) -> None: ...
    @property
    def started_at(self) -> datetime: ...
    @started_at.setter
    def started_at(self, started_at: datetime) -> None: ...



from kubernetes_asyncio.client.models import V1ObjectReference
from typing import Any

class V1EndpointAddress:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, hostname: str | None = None, ip: str | None = None, node_name: str | None = None, target_ref: V1ObjectReference | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def hostname(self) -> str: ...
    @hostname.setter
    def hostname(self, hostname: str) -> None: ...
    @property
    def ip(self) -> str: ...
    @ip.setter
    def ip(self, ip: str) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def target_ref(self) -> V1ObjectReference: ...
    @target_ref.setter
    def target_ref(self, target_ref: V1ObjectReference) -> None: ...



from kubernetes_asyncio.client.models import V1EndpointConditions
from kubernetes_asyncio.client.models import V1EndpointHints
from kubernetes_asyncio.client.models import V1ObjectReference
from typing import Any

class V1Endpoint:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, addresses: list[str] | None = None, conditions: V1EndpointConditions | None = None, deprecated_topology: dict[str, str] | None = None, hints: V1EndpointHints | None = None, hostname: str | None = None, node_name: str | None = None, target_ref: V1ObjectReference | None = None, zone: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def addresses(self) -> list[str]: ...
    @addresses.setter
    def addresses(self, addresses: list[str]) -> None: ...
    @property
    def conditions(self) -> V1EndpointConditions: ...
    @conditions.setter
    def conditions(self, conditions: V1EndpointConditions) -> None: ...
    @property
    def deprecated_topology(self) -> dict[str, str]: ...
    @deprecated_topology.setter
    def deprecated_topology(self, deprecated_topology: dict[str, str]) -> None: ...
    @property
    def hints(self) -> V1EndpointHints: ...
    @hints.setter
    def hints(self, hints: V1EndpointHints) -> None: ...
    @property
    def hostname(self) -> str: ...
    @hostname.setter
    def hostname(self, hostname: str) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def target_ref(self) -> V1ObjectReference: ...
    @target_ref.setter
    def target_ref(self, target_ref: V1ObjectReference) -> None: ...
    @property
    def zone(self) -> str: ...
    @zone.setter
    def zone(self, zone: str) -> None: ...



from typing import Any

class V1beta1NetworkDeviceData:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, hardware_address: str | None = None, interface_name: str | None = None, ips: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def hardware_address(self) -> str: ...
    @hardware_address.setter
    def hardware_address(self, hardware_address: str) -> None: ...
    @property
    def interface_name(self) -> str: ...
    @interface_name.setter
    def interface_name(self, interface_name: str) -> None: ...
    @property
    def ips(self) -> list[str]: ...
    @ips.setter
    def ips(self, ips: list[str]) -> None: ...



from kubernetes_asyncio.client.models import AdmissionregistrationV1WebhookClientConfig
from kubernetes_asyncio.client.models import V1LabelSelector
from kubernetes_asyncio.client.models import V1MatchCondition
from kubernetes_asyncio.client.models import V1RuleWithOperations
from typing import Any

class V1ValidatingWebhook:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, admission_review_versions: list[str] | None = None, client_config: AdmissionregistrationV1WebhookClientConfig | None = None, failure_policy: str | None = None, match_conditions: list[V1MatchCondition] | None = None, match_policy: str | None = None, name: str | None = None, namespace_selector: V1LabelSelector | None = None, object_selector: V1LabelSelector | None = None, rules: list[V1RuleWithOperations] | None = None, side_effects: str | None = None, timeout_seconds: int | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def admission_review_versions(self) -> list[str]: ...
    @admission_review_versions.setter
    def admission_review_versions(self, admission_review_versions: list[str]) -> None: ...
    @property
    def client_config(self) -> AdmissionregistrationV1WebhookClientConfig: ...
    @client_config.setter
    def client_config(self, client_config: AdmissionregistrationV1WebhookClientConfig) -> None: ...
    @property
    def failure_policy(self) -> str: ...
    @failure_policy.setter
    def failure_policy(self, failure_policy: str) -> None: ...
    @property
    def match_conditions(self) -> list[V1MatchCondition]: ...
    @match_conditions.setter
    def match_conditions(self, match_conditions: list[V1MatchCondition]) -> None: ...
    @property
    def match_policy(self) -> str: ...
    @match_policy.setter
    def match_policy(self, match_policy: str) -> None: ...
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, name: str) -> None: ...
    @property
    def namespace_selector(self) -> V1LabelSelector: ...
    @namespace_selector.setter
    def namespace_selector(self, namespace_selector: V1LabelSelector) -> None: ...
    @property
    def object_selector(self) -> V1LabelSelector: ...
    @object_selector.setter
    def object_selector(self, object_selector: V1LabelSelector) -> None: ...
    @property
    def rules(self) -> list[V1RuleWithOperations]: ...
    @rules.setter
    def rules(self, rules: list[V1RuleWithOperations]) -> None: ...
    @property
    def side_effects(self) -> str: ...
    @side_effects.setter
    def side_effects(self, side_effects: str) -> None: ...
    @property
    def timeout_seconds(self) -> int: ...
    @timeout_seconds.setter
    def timeout_seconds(self, timeout_seconds: int) -> None: ...



# Copyright 2016 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import warnings
from os.path import exists, expanduser
from typing import Any

from kubernetes_asyncio.config.config_exception import ConfigException
from kubernetes_asyncio.config.incluster_config import load_incluster_config
from kubernetes_asyncio.config.kube_config import (
    KUBE_CONFIG_DEFAULT_LOCATION, list_kube_config_contexts, load_kube_config,
    load_kube_config_from_dict, new_client_from_config,
    new_client_from_config_dict,
)

__all__ = [
    "ConfigException",
    "list_kube_config_contexts",
    "load_kube_config_from_dict",
    "new_client_from_config",
    "new_client_from_config_dict",
]


async def load_config(**kwargs: Any) -> None:
    """
    Wrapper function to load the kube_config.
    It will initially try to load_kube_config from provided path,
    then check if the KUBE_CONFIG_DEFAULT_LOCATION exists
    If neither exists, it will fall back to load_incluster_config
    and inform the user accordingly.

    :param kwargs: A combination of all possible kwargs that
    can be passed to either load_kube_config or
    load_incluster_config functions.
    """
    if "config_file" in kwargs.keys():
        await load_kube_config(**kwargs)
    elif "kube_config_path" in kwargs.keys():
        kwargs["config_file"] = kwargs.pop("kube_config_path", None)
        await load_kube_config(**kwargs)
    elif exists(expanduser(KUBE_CONFIG_DEFAULT_LOCATION)):
        await load_kube_config(**kwargs)
    else:
        warnings.warn(
            "kube_config_path not provided and "
            "default location ({0}) does not exist. "
            "Using inCluster Config. "
            "This might not work.".format(KUBE_CONFIG_DEFAULT_LOCATION)
        )
        load_incluster_config(**kwargs)



from kubernetes_asyncio.client.models import V1LabelSelector
from typing import Any

class V1PodDisruptionBudgetSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, max_unavailable: Any | None = None, min_available: Any | None = None, selector: V1LabelSelector | None = None, unhealthy_pod_eviction_policy: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def max_unavailable(self) -> Any: ...
    @max_unavailable.setter
    def max_unavailable(self, max_unavailable: Any) -> None: ...
    @property
    def min_available(self) -> Any: ...
    @min_available.setter
    def min_available(self, min_available: Any) -> None: ...
    @property
    def selector(self) -> V1LabelSelector: ...
    @selector.setter
    def selector(self, selector: V1LabelSelector) -> None: ...
    @property
    def unhealthy_pod_eviction_policy(self) -> str: ...
    @unhealthy_pod_eviction_policy.setter
    def unhealthy_pod_eviction_policy(self, unhealthy_pod_eviction_policy: str) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1LocalSubjectAccessReview
from kubernetes_asyncio.client.models import V1SelfSubjectAccessReview
from kubernetes_asyncio.client.models import V1SelfSubjectRulesReview
from kubernetes_asyncio.client.models import V1SubjectAccessReview
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class AuthorizationV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_namespaced_local_subject_access_review(self, namespace: str, body: V1LocalSubjectAccessReview, **kwargs: Any) -> Awaitable[V1LocalSubjectAccessReview]: ...
    def create_namespaced_local_subject_access_review_with_http_info(self, namespace: str, body: V1LocalSubjectAccessReview, **kwargs: Any) -> Awaitable[tuple[V1LocalSubjectAccessReview, int, CIMultiDictProxy]]: ...
    def create_self_subject_access_review(self, body: V1SelfSubjectAccessReview, **kwargs: Any) -> Awaitable[V1SelfSubjectAccessReview]: ...
    def create_self_subject_access_review_with_http_info(self, body: V1SelfSubjectAccessReview, **kwargs: Any) -> Awaitable[tuple[V1SelfSubjectAccessReview, int, CIMultiDictProxy]]: ...
    def create_self_subject_rules_review(self, body: V1SelfSubjectRulesReview, **kwargs: Any) -> Awaitable[V1SelfSubjectRulesReview]: ...
    def create_self_subject_rules_review_with_http_info(self, body: V1SelfSubjectRulesReview, **kwargs: Any) -> Awaitable[tuple[V1SelfSubjectRulesReview, int, CIMultiDictProxy]]: ...
    def create_subject_access_review(self, body: V1SubjectAccessReview, **kwargs: Any) -> Awaitable[V1SubjectAccessReview]: ...
    def create_subject_access_review_with_http_info(self, body: V1SubjectAccessReview, **kwargs: Any) -> Awaitable[tuple[V1SubjectAccessReview, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...



from datetime import datetime
from kubernetes_asyncio.client.models import V1Condition
from typing import Any

class V1alpha1PodCertificateRequestStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, begin_refresh_at: datetime | None = None, certificate_chain: str | None = None, conditions: list[V1Condition] | None = None, not_after: datetime | None = None, not_before: datetime | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def begin_refresh_at(self) -> datetime: ...
    @begin_refresh_at.setter
    def begin_refresh_at(self, begin_refresh_at: datetime) -> None: ...
    @property
    def certificate_chain(self) -> str: ...
    @certificate_chain.setter
    def certificate_chain(self, certificate_chain: str) -> None: ...
    @property
    def conditions(self) -> list[V1Condition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1Condition]) -> None: ...
    @property
    def not_after(self) -> datetime: ...
    @not_after.setter
    def not_after(self, not_after: datetime) -> None: ...
    @property
    def not_before(self) -> datetime: ...
    @not_before.setter
    def not_before(self, not_before: datetime) -> None: ...



from kubernetes_asyncio.client.models import V1DeviceClaimConfiguration
from kubernetes_asyncio.client.models import V1DeviceConstraint
from kubernetes_asyncio.client.models import V1DeviceRequest
from typing import Any

class V1DeviceClaim:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config: list[V1DeviceClaimConfiguration] | None = None, constraints: list[V1DeviceConstraint] | None = None, requests: list[V1DeviceRequest] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config(self) -> list[V1DeviceClaimConfiguration]: ...
    @config.setter
    def config(self, config: list[V1DeviceClaimConfiguration]) -> None: ...
    @property
    def constraints(self) -> list[V1DeviceConstraint]: ...
    @constraints.setter
    def constraints(self, constraints: list[V1DeviceConstraint]) -> None: ...
    @property
    def requests(self) -> list[V1DeviceRequest]: ...
    @requests.setter
    def requests(self, requests: list[V1DeviceRequest]) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1MutatingWebhookConfiguration
from kubernetes_asyncio.client.models import V1MutatingWebhookConfigurationList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1ValidatingAdmissionPolicy
from kubernetes_asyncio.client.models import V1ValidatingAdmissionPolicyBinding
from kubernetes_asyncio.client.models import V1ValidatingAdmissionPolicyBindingList
from kubernetes_asyncio.client.models import V1ValidatingAdmissionPolicyList
from kubernetes_asyncio.client.models import V1ValidatingWebhookConfiguration
from kubernetes_asyncio.client.models import V1ValidatingWebhookConfigurationList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class AdmissionregistrationV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_mutating_webhook_configuration(self, body: V1MutatingWebhookConfiguration, **kwargs: Any) -> Awaitable[V1MutatingWebhookConfiguration]: ...
    def create_mutating_webhook_configuration_with_http_info(self, body: V1MutatingWebhookConfiguration, **kwargs: Any) -> Awaitable[tuple[V1MutatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def create_validating_admission_policy(self, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def create_validating_admission_policy_binding(self, body: V1ValidatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyBinding]: ...
    def create_validating_admission_policy_binding_with_http_info(self, body: V1ValidatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def create_validating_admission_policy_with_http_info(self, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def create_validating_webhook_configuration(self, body: V1ValidatingWebhookConfiguration, **kwargs: Any) -> Awaitable[V1ValidatingWebhookConfiguration]: ...
    def create_validating_webhook_configuration_with_http_info(self, body: V1ValidatingWebhookConfiguration, **kwargs: Any) -> Awaitable[tuple[V1ValidatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def delete_collection_mutating_webhook_configuration(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_mutating_webhook_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_validating_admission_policy(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_validating_admission_policy_binding(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_validating_admission_policy_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_validating_admission_policy_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_validating_webhook_configuration(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_validating_webhook_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_mutating_webhook_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_mutating_webhook_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_validating_admission_policy(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_validating_admission_policy_binding(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_validating_admission_policy_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_validating_admission_policy_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_validating_webhook_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_validating_webhook_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_mutating_webhook_configuration(self, **kwargs: Any) -> Awaitable[V1MutatingWebhookConfigurationList]: ...
    def list_mutating_webhook_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1MutatingWebhookConfigurationList, int, CIMultiDictProxy]]: ...
    def list_validating_admission_policy(self, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyList]: ...
    def list_validating_admission_policy_binding(self, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyBindingList]: ...
    def list_validating_admission_policy_binding_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyBindingList, int, CIMultiDictProxy]]: ...
    def list_validating_admission_policy_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyList, int, CIMultiDictProxy]]: ...
    def list_validating_webhook_configuration(self, **kwargs: Any) -> Awaitable[V1ValidatingWebhookConfigurationList]: ...
    def list_validating_webhook_configuration_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1ValidatingWebhookConfigurationList, int, CIMultiDictProxy]]: ...
    def patch_mutating_webhook_configuration(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1MutatingWebhookConfiguration]: ...
    def patch_mutating_webhook_configuration_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1MutatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def patch_validating_admission_policy(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def patch_validating_admission_policy_binding(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyBinding]: ...
    def patch_validating_admission_policy_binding_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def patch_validating_admission_policy_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def patch_validating_admission_policy_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def patch_validating_admission_policy_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def patch_validating_webhook_configuration(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1ValidatingWebhookConfiguration]: ...
    def patch_validating_webhook_configuration_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1ValidatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def read_mutating_webhook_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1MutatingWebhookConfiguration]: ...
    def read_mutating_webhook_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1MutatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def read_validating_admission_policy(self, name: str, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def read_validating_admission_policy_binding(self, name: str, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyBinding]: ...
    def read_validating_admission_policy_binding_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def read_validating_admission_policy_status(self, name: str, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def read_validating_admission_policy_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def read_validating_admission_policy_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def read_validating_webhook_configuration(self, name: str, **kwargs: Any) -> Awaitable[V1ValidatingWebhookConfiguration]: ...
    def read_validating_webhook_configuration_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1ValidatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def replace_mutating_webhook_configuration(self, name: str, body: V1MutatingWebhookConfiguration, **kwargs: Any) -> Awaitable[V1MutatingWebhookConfiguration]: ...
    def replace_mutating_webhook_configuration_with_http_info(self, name: str, body: V1MutatingWebhookConfiguration, **kwargs: Any) -> Awaitable[tuple[V1MutatingWebhookConfiguration, int, CIMultiDictProxy]]: ...
    def replace_validating_admission_policy(self, name: str, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def replace_validating_admission_policy_binding(self, name: str, body: V1ValidatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicyBinding]: ...
    def replace_validating_admission_policy_binding_with_http_info(self, name: str, body: V1ValidatingAdmissionPolicyBinding, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicyBinding, int, CIMultiDictProxy]]: ...
    def replace_validating_admission_policy_status(self, name: str, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[V1ValidatingAdmissionPolicy]: ...
    def replace_validating_admission_policy_status_with_http_info(self, name: str, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def replace_validating_admission_policy_with_http_info(self, name: str, body: V1ValidatingAdmissionPolicy, **kwargs: Any) -> Awaitable[tuple[V1ValidatingAdmissionPolicy, int, CIMultiDictProxy]]: ...
    def replace_validating_webhook_configuration(self, name: str, body: V1ValidatingWebhookConfiguration, **kwargs: Any) -> Awaitable[V1ValidatingWebhookConfiguration]: ...
    def replace_validating_webhook_configuration_with_http_info(self, name: str, body: V1ValidatingWebhookConfiguration, **kwargs: Any) -> Awaitable[tuple[V1ValidatingWebhookConfiguration, int, CIMultiDictProxy]]: ...



import unittest
from kubernetes_asyncio.client.configuration import Configuration as Configuration
from kubernetes_asyncio.client.rest import RESTClientObject as RESTClientObject

class TestRESTClientObject(unittest.IsolatedAsyncioTestCase):
    @classmethod
    def setUpClass(cls) -> None: ...
    async def test_rest_request_timeout(self) -> None: ...
    async def test_disable_ssl_verification(self) -> None: ...



from kubernetes_asyncio.client.api_client import ApiClient
from kubernetes_asyncio.client.models import V1APIResourceList
from kubernetes_asyncio.client.models import V1CSIDriver
from kubernetes_asyncio.client.models import V1CSIDriverList
from kubernetes_asyncio.client.models import V1CSINode
from kubernetes_asyncio.client.models import V1CSINodeList
from kubernetes_asyncio.client.models import V1CSIStorageCapacity
from kubernetes_asyncio.client.models import V1CSIStorageCapacityList
from kubernetes_asyncio.client.models import V1Status
from kubernetes_asyncio.client.models import V1StorageClass
from kubernetes_asyncio.client.models import V1StorageClassList
from kubernetes_asyncio.client.models import V1VolumeAttachment
from kubernetes_asyncio.client.models import V1VolumeAttachmentList
from kubernetes_asyncio.client.models import V1VolumeAttributesClass
from kubernetes_asyncio.client.models import V1VolumeAttributesClassList
from multidict import CIMultiDictProxy
from typing import Any
from typing import Awaitable

class StorageV1Api:
    def __init__(self, api_client: ApiClient | None = None) -> None: ...
    def create_csi_driver(self, body: V1CSIDriver, **kwargs: Any) -> Awaitable[V1CSIDriver]: ...
    def create_csi_driver_with_http_info(self, body: V1CSIDriver, **kwargs: Any) -> Awaitable[tuple[V1CSIDriver, int, CIMultiDictProxy]]: ...
    def create_csi_node(self, body: V1CSINode, **kwargs: Any) -> Awaitable[V1CSINode]: ...
    def create_csi_node_with_http_info(self, body: V1CSINode, **kwargs: Any) -> Awaitable[tuple[V1CSINode, int, CIMultiDictProxy]]: ...
    def create_namespaced_csi_storage_capacity(self, namespace: str, body: V1CSIStorageCapacity, **kwargs: Any) -> Awaitable[V1CSIStorageCapacity]: ...
    def create_namespaced_csi_storage_capacity_with_http_info(self, namespace: str, body: V1CSIStorageCapacity, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacity, int, CIMultiDictProxy]]: ...
    def create_storage_class(self, body: V1StorageClass, **kwargs: Any) -> Awaitable[V1StorageClass]: ...
    def create_storage_class_with_http_info(self, body: V1StorageClass, **kwargs: Any) -> Awaitable[tuple[V1StorageClass, int, CIMultiDictProxy]]: ...
    def create_volume_attachment(self, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def create_volume_attachment_with_http_info(self, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def create_volume_attributes_class(self, body: V1VolumeAttributesClass, **kwargs: Any) -> Awaitable[V1VolumeAttributesClass]: ...
    def create_volume_attributes_class_with_http_info(self, body: V1VolumeAttributesClass, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def delete_collection_csi_driver(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_csi_driver_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_csi_node(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_csi_node_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_namespaced_csi_storage_capacity(self, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_namespaced_csi_storage_capacity_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_storage_class(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_storage_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_volume_attachment(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_volume_attachment_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_collection_volume_attributes_class(self, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_collection_volume_attributes_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_csi_driver(self, name: str, **kwargs: Any) -> Awaitable[V1CSIDriver]: ...
    def delete_csi_driver_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CSIDriver, int, CIMultiDictProxy]]: ...
    def delete_csi_node(self, name: str, **kwargs: Any) -> Awaitable[V1CSINode]: ...
    def delete_csi_node_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CSINode, int, CIMultiDictProxy]]: ...
    def delete_namespaced_csi_storage_capacity(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1Status]: ...
    def delete_namespaced_csi_storage_capacity_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1Status, int, CIMultiDictProxy]]: ...
    def delete_storage_class(self, name: str, **kwargs: Any) -> Awaitable[V1StorageClass]: ...
    def delete_storage_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1StorageClass, int, CIMultiDictProxy]]: ...
    def delete_volume_attachment(self, name: str, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def delete_volume_attachment_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def delete_volume_attributes_class(self, name: str, **kwargs: Any) -> Awaitable[V1VolumeAttributesClass]: ...
    def delete_volume_attributes_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def get_api_resources(self, **kwargs: Any) -> Awaitable[V1APIResourceList]: ...
    def get_api_resources_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1APIResourceList, int, CIMultiDictProxy]]: ...
    def list_csi_driver(self, **kwargs: Any) -> Awaitable[V1CSIDriverList]: ...
    def list_csi_driver_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1CSIDriverList, int, CIMultiDictProxy]]: ...
    def list_csi_node(self, **kwargs: Any) -> Awaitable[V1CSINodeList]: ...
    def list_csi_node_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1CSINodeList, int, CIMultiDictProxy]]: ...
    def list_csi_storage_capacity_for_all_namespaces(self, **kwargs: Any) -> Awaitable[V1CSIStorageCapacityList]: ...
    def list_csi_storage_capacity_for_all_namespaces_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacityList, int, CIMultiDictProxy]]: ...
    def list_namespaced_csi_storage_capacity(self, namespace: str, **kwargs: Any) -> Awaitable[V1CSIStorageCapacityList]: ...
    def list_namespaced_csi_storage_capacity_with_http_info(self, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacityList, int, CIMultiDictProxy]]: ...
    def list_storage_class(self, **kwargs: Any) -> Awaitable[V1StorageClassList]: ...
    def list_storage_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1StorageClassList, int, CIMultiDictProxy]]: ...
    def list_volume_attachment(self, **kwargs: Any) -> Awaitable[V1VolumeAttachmentList]: ...
    def list_volume_attachment_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachmentList, int, CIMultiDictProxy]]: ...
    def list_volume_attributes_class(self, **kwargs: Any) -> Awaitable[V1VolumeAttributesClassList]: ...
    def list_volume_attributes_class_with_http_info(self, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClassList, int, CIMultiDictProxy]]: ...
    def patch_csi_driver(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1CSIDriver]: ...
    def patch_csi_driver_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CSIDriver, int, CIMultiDictProxy]]: ...
    def patch_csi_node(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1CSINode]: ...
    def patch_csi_node_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CSINode, int, CIMultiDictProxy]]: ...
    def patch_namespaced_csi_storage_capacity(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[V1CSIStorageCapacity]: ...
    def patch_namespaced_csi_storage_capacity_with_http_info(self, name: str, namespace: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacity, int, CIMultiDictProxy]]: ...
    def patch_storage_class(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1StorageClass]: ...
    def patch_storage_class_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1StorageClass, int, CIMultiDictProxy]]: ...
    def patch_volume_attachment(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def patch_volume_attachment_status(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def patch_volume_attachment_status_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def patch_volume_attachment_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def patch_volume_attributes_class(self, name: str, body: Any, **kwargs: Any) -> Awaitable[V1VolumeAttributesClass]: ...
    def patch_volume_attributes_class_with_http_info(self, name: str, body: Any, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def read_csi_driver(self, name: str, **kwargs: Any) -> Awaitable[V1CSIDriver]: ...
    def read_csi_driver_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CSIDriver, int, CIMultiDictProxy]]: ...
    def read_csi_node(self, name: str, **kwargs: Any) -> Awaitable[V1CSINode]: ...
    def read_csi_node_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1CSINode, int, CIMultiDictProxy]]: ...
    def read_namespaced_csi_storage_capacity(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[V1CSIStorageCapacity]: ...
    def read_namespaced_csi_storage_capacity_with_http_info(self, name: str, namespace: str, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacity, int, CIMultiDictProxy]]: ...
    def read_storage_class(self, name: str, **kwargs: Any) -> Awaitable[V1StorageClass]: ...
    def read_storage_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1StorageClass, int, CIMultiDictProxy]]: ...
    def read_volume_attachment(self, name: str, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def read_volume_attachment_status(self, name: str, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def read_volume_attachment_status_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def read_volume_attachment_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def read_volume_attributes_class(self, name: str, **kwargs: Any) -> Awaitable[V1VolumeAttributesClass]: ...
    def read_volume_attributes_class_with_http_info(self, name: str, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClass, int, CIMultiDictProxy]]: ...
    def replace_csi_driver(self, name: str, body: V1CSIDriver, **kwargs: Any) -> Awaitable[V1CSIDriver]: ...
    def replace_csi_driver_with_http_info(self, name: str, body: V1CSIDriver, **kwargs: Any) -> Awaitable[tuple[V1CSIDriver, int, CIMultiDictProxy]]: ...
    def replace_csi_node(self, name: str, body: V1CSINode, **kwargs: Any) -> Awaitable[V1CSINode]: ...
    def replace_csi_node_with_http_info(self, name: str, body: V1CSINode, **kwargs: Any) -> Awaitable[tuple[V1CSINode, int, CIMultiDictProxy]]: ...
    def replace_namespaced_csi_storage_capacity(self, name: str, namespace: str, body: V1CSIStorageCapacity, **kwargs: Any) -> Awaitable[V1CSIStorageCapacity]: ...
    def replace_namespaced_csi_storage_capacity_with_http_info(self, name: str, namespace: str, body: V1CSIStorageCapacity, **kwargs: Any) -> Awaitable[tuple[V1CSIStorageCapacity, int, CIMultiDictProxy]]: ...
    def replace_storage_class(self, name: str, body: V1StorageClass, **kwargs: Any) -> Awaitable[V1StorageClass]: ...
    def replace_storage_class_with_http_info(self, name: str, body: V1StorageClass, **kwargs: Any) -> Awaitable[tuple[V1StorageClass, int, CIMultiDictProxy]]: ...
    def replace_volume_attachment(self, name: str, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def replace_volume_attachment_status(self, name: str, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[V1VolumeAttachment]: ...
    def replace_volume_attachment_status_with_http_info(self, name: str, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def replace_volume_attachment_with_http_info(self, name: str, body: V1VolumeAttachment, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttachment, int, CIMultiDictProxy]]: ...
    def replace_volume_attributes_class(self, name: str, body: V1VolumeAttributesClass, **kwargs: Any) -> Awaitable[V1VolumeAttributesClass]: ...
    def replace_volume_attributes_class_with_http_info(self, name: str, body: V1VolumeAttributesClass, **kwargs: Any) -> Awaitable[tuple[V1VolumeAttributesClass, int, CIMultiDictProxy]]: ...



from kubernetes_asyncio.client.models import V1ConfigMapKeySelector
from kubernetes_asyncio.client.models import V1FileKeySelector
from kubernetes_asyncio.client.models import V1ObjectFieldSelector
from kubernetes_asyncio.client.models import V1ResourceFieldSelector
from kubernetes_asyncio.client.models import V1SecretKeySelector
from typing import Any

class V1EnvVarSource:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, config_map_key_ref: V1ConfigMapKeySelector | None = None, field_ref: V1ObjectFieldSelector | None = None, file_key_ref: V1FileKeySelector | None = None, resource_field_ref: V1ResourceFieldSelector | None = None, secret_key_ref: V1SecretKeySelector | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def config_map_key_ref(self) -> V1ConfigMapKeySelector: ...
    @config_map_key_ref.setter
    def config_map_key_ref(self, config_map_key_ref: V1ConfigMapKeySelector) -> None: ...
    @property
    def field_ref(self) -> V1ObjectFieldSelector: ...
    @field_ref.setter
    def field_ref(self, field_ref: V1ObjectFieldSelector) -> None: ...
    @property
    def file_key_ref(self) -> V1FileKeySelector: ...
    @file_key_ref.setter
    def file_key_ref(self, file_key_ref: V1FileKeySelector) -> None: ...
    @property
    def resource_field_ref(self) -> V1ResourceFieldSelector: ...
    @resource_field_ref.setter
    def resource_field_ref(self, resource_field_ref: V1ResourceFieldSelector) -> None: ...
    @property
    def secret_key_ref(self) -> V1SecretKeySelector: ...
    @secret_key_ref.setter
    def secret_key_ref(self, secret_key_ref: V1SecretKeySelector) -> None: ...



from kubernetes_asyncio.client.models import V1Toleration
from typing import Any

class V1Scheduling:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, node_selector: dict[str, str] | None = None, tolerations: list[V1Toleration] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def node_selector(self) -> dict[str, str]: ...
    @node_selector.setter
    def node_selector(self, node_selector: dict[str, str]) -> None: ...
    @property
    def tolerations(self) -> list[V1Toleration]: ...
    @tolerations.setter
    def tolerations(self, tolerations: list[V1Toleration]) -> None: ...



from typing import Any

class V1EndpointConditions:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, ready: bool | None = None, serving: bool | None = None, terminating: bool | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def ready(self) -> bool: ...
    @ready.setter
    def ready(self, ready: bool) -> None: ...
    @property
    def serving(self) -> bool: ...
    @serving.setter
    def serving(self, serving: bool) -> None: ...
    @property
    def terminating(self) -> bool: ...
    @terminating.setter
    def terminating(self, terminating: bool) -> None: ...



from kubernetes_asyncio.client.models import StorageV1TokenRequest
from typing import Any

class V1CSIDriverSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, attach_required: bool | None = None, fs_group_policy: str | None = None, node_allocatable_update_period_seconds: int | None = None, pod_info_on_mount: bool | None = None, requires_republish: bool | None = None, se_linux_mount: bool | None = None, storage_capacity: bool | None = None, token_requests: list[StorageV1TokenRequest] | None = None, volume_lifecycle_modes: list[str] | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def attach_required(self) -> bool: ...
    @attach_required.setter
    def attach_required(self, attach_required: bool) -> None: ...
    @property
    def fs_group_policy(self) -> str: ...
    @fs_group_policy.setter
    def fs_group_policy(self, fs_group_policy: str) -> None: ...
    @property
    def node_allocatable_update_period_seconds(self) -> int: ...
    @node_allocatable_update_period_seconds.setter
    def node_allocatable_update_period_seconds(self, node_allocatable_update_period_seconds: int) -> None: ...
    @property
    def pod_info_on_mount(self) -> bool: ...
    @pod_info_on_mount.setter
    def pod_info_on_mount(self, pod_info_on_mount: bool) -> None: ...
    @property
    def requires_republish(self) -> bool: ...
    @requires_republish.setter
    def requires_republish(self, requires_republish: bool) -> None: ...
    @property
    def se_linux_mount(self) -> bool: ...
    @se_linux_mount.setter
    def se_linux_mount(self, se_linux_mount: bool) -> None: ...
    @property
    def storage_capacity(self) -> bool: ...
    @storage_capacity.setter
    def storage_capacity(self, storage_capacity: bool) -> None: ...
    @property
    def token_requests(self) -> list[StorageV1TokenRequest]: ...
    @token_requests.setter
    def token_requests(self, token_requests: list[StorageV1TokenRequest]) -> None: ...
    @property
    def volume_lifecycle_modes(self) -> list[str]: ...
    @volume_lifecycle_modes.setter
    def volume_lifecycle_modes(self, volume_lifecycle_modes: list[str]) -> None: ...



from typing import Any

class V1alpha1PodCertificateRequestSpec:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, max_expiration_seconds: int | None = None, node_name: str | None = None, node_uid: str | None = None, pkix_public_key: str | None = None, pod_name: str | None = None, pod_uid: str | None = None, proof_of_possession: str | None = None, service_account_name: str | None = None, service_account_uid: str | None = None, signer_name: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def max_expiration_seconds(self) -> int: ...
    @max_expiration_seconds.setter
    def max_expiration_seconds(self, max_expiration_seconds: int) -> None: ...
    @property
    def node_name(self) -> str: ...
    @node_name.setter
    def node_name(self, node_name: str) -> None: ...
    @property
    def node_uid(self) -> str: ...
    @node_uid.setter
    def node_uid(self, node_uid: str) -> None: ...
    @property
    def pkix_public_key(self) -> str: ...
    @pkix_public_key.setter
    def pkix_public_key(self, pkix_public_key: str) -> None: ...
    @property
    def pod_name(self) -> str: ...
    @pod_name.setter
    def pod_name(self, pod_name: str) -> None: ...
    @property
    def pod_uid(self) -> str: ...
    @pod_uid.setter
    def pod_uid(self, pod_uid: str) -> None: ...
    @property
    def proof_of_possession(self) -> str: ...
    @proof_of_possession.setter
    def proof_of_possession(self, proof_of_possession: str) -> None: ...
    @property
    def service_account_name(self) -> str: ...
    @service_account_name.setter
    def service_account_name(self, service_account_name: str) -> None: ...
    @property
    def service_account_uid(self) -> str: ...
    @service_account_uid.setter
    def service_account_uid(self, service_account_uid: str) -> None: ...
    @property
    def signer_name(self) -> str: ...
    @signer_name.setter
    def signer_name(self, signer_name: str) -> None: ...



from kubernetes_asyncio.client.models import V1ModifyVolumeStatus
from kubernetes_asyncio.client.models import V1PersistentVolumeClaimCondition
from typing import Any

class V1PersistentVolumeClaimStatus:
    def __eq__(self, other: Any) -> bool: ...
    def __init__(self, access_modes: list[str] | None = None, allocated_resource_statuses: dict[str, str] | None = None, allocated_resources: dict[str, str] | None = None, capacity: dict[str, str] | None = None, conditions: list[V1PersistentVolumeClaimCondition] | None = None, current_volume_attributes_class_name: str | None = None, modify_volume_status: V1ModifyVolumeStatus | None = None, phase: str | None = None, local_vars_configuration: Any | None = None) -> None: ...
    def __ne__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...
    def to_dict(self, serialize: bool = False) -> dict[str, Any]: ...
    def to_str(self) -> str: ...
    @property
    def access_modes(self) -> list[str]: ...
    @access_modes.setter
    def access_modes(self, access_modes: list[str]) -> None: ...
    @property
    def allocated_resource_statuses(self) -> dict[str, str]: ...
    @allocated_resource_statuses.setter
    def allocated_resource_statuses(self, allocated_resource_statuses: dict[str, str]) -> None: ...
    @property
    def allocated_resources(self) -> dict[str, str]: ...
    @allocated_resources.setter
    def allocated_resources(self, allocated_resources: dict[str, str]) -> None: ...
    @property
    def capacity(self) -> dict[str, str]: ...
    @capacity.setter
    def capacity(self, capacity: dict[str, str]) -> None: ...
    @property
    def conditions(self) -> list[V1PersistentVolumeClaimCondition]: ...
    @conditions.setter
    def conditions(self, conditions: list[V1PersistentVolumeClaimCondition]) -> None: ...
    @property
    def current_volume_attributes_class_name(self) -> str: ...
    @current_volume_attributes_class_name.setter
    def current_volume_attributes_class_name(self, current_volume_attributes_class_name: str) -> None: ...
    @property
    def modify_volume_status(self) -> V1ModifyVolumeStatus: ...
    @modify_volume_status.setter
    def modify_volume_status(self, modify_volume_status: V1ModifyVolumeStatus) -> None: ...
    @property
    def phase(self) -> str: ...
    @phase.setter
    def phase(self, phase: str) -> None: ...
